@@ -5,7 +5,6 @@ import numpy as np
 from bert4keras.layers import *
 from keras.models import Model
 import json
-import warnings
 
 
 class Transformer(object):
@@ -19,7 +18,7 @@ class Transformer(object):
             num_attention_heads,  # Attention的头数
             intermediate_size,  # FeedForward的隐层维度
             hidden_act,  # FeedForward隐层的激活函数
-            dropout_rate,  # Dropout比例
+            dropout_rate=None,  # Dropout比例
             embedding_size=None,  # 是否指定embedding_size
             attention_key_size=None,  # Attention中Q,K的head_size
             keep_tokens=None,  # 要保留的词ID列表
@@ -36,7 +35,7 @@ class Transformer(object):
         self.attention_head_size = hidden_size // num_attention_heads
         self.attention_key_size = attention_key_size or self.attention_head_size
         self.intermediate_size = intermediate_size
-        self.dropout_rate = dropout_rate
+        self.dropout_rate = dropout_rate or 0
         self.hidden_act = hidden_act
         self.embedding_size = embedding_size or hidden_size
         self.keep_tokens = keep_tokens
@@ -184,6 +183,7 @@ class Transformer(object):
         """根据mapping从checkpoint加载权重
         """
         mapping = mapping or self.variable_mapping()
+        mapping = {k: v for k, v in mapping.items() if k in self.layers}
 
         weight_value_pairs = []
         for layer, variables in mapping.items():
@@ -523,8 +523,6 @@ class BERT(Transformer):
                 ],
             })
 
-        mapping = {k: v for k, v in mapping.items() if k in self.layers}
-
         return mapping
 
 
@@ -629,8 +627,6 @@ class ALBERT(BERT):
             ],
         })
 
-        mapping = {k: v for k, v in mapping.items() if k in self.layers}
-
         return mapping
 
 
@@ -671,8 +667,6 @@ class ALBERT_Unshared(BERT):
                 ],
             })
 
-        mapping = {k: v for k, v in mapping.items() if k in self.layers}
-
         return mapping
 
 
@@ -1057,8 +1051,6 @@ class GPT2_ML(Transformer):
                 ],
             })
 
-        mapping = {k: v for k, v in mapping.items() if k in self.layers}
-
         return mapping
 
 
@@ -1152,8 +1144,6 @@ class T5_Base(Transformer):
                 ],
             })
 
-        mapping = {k: v for k, v in mapping.items() if k in self.layers}
-
         return mapping
 
 
@@ -1673,13 +1663,3 @@ def build_transformer_model(config_path=None,
         return transformer.model
     else:
         return transformer
-
-
-def build_bert_model(*args, **kwargs):
-    warnings.warn('build_bert_model has been renamed as build_transformer_model.')
-    warnings.warn('please use build_transformer_model.')
-    if kwargs.get('application') == 'seq2seq':
-        warnings.warn('application=\'seq2seq\' has been renamed as application=\'unilm\'')
-        warnings.warn('please use application=\'unilm\'')
-        kwargs['application'] = 'unilm'
-    return build_transformer_model(*args, **kwargs)
