@@ -295,7 +295,25 @@ class AdaFactorV2(AdaFactorBase):
         return self._resource_apply_dense(grad, var)
 
 
-def extend_with_weight_decay(base_optimizer, name=None):
+def export_to_custom_objects(base_extend_with):
+    """装饰器，用来将优化器放到custom_objects中
+    """
+    def new_extend_with(base_optimizer, name=None):
+        new_optimizer = base_extend_with(base_optimizer)
+
+        if is_string(name):
+            new_optimizer.__name__ = name
+
+        name = new_optimizer.__name__
+        keras.utils.get_custom_objects()[name] = new_optimizer
+
+        return new_optimizer
+
+    return new_extend_with
+
+
+@export_to_custom_objects
+def extend_with_weight_decay(base_optimizer):
     """返回新的优化器类，加入权重衰减
     """
     class new_optimizer(base_optimizer):
@@ -339,14 +357,11 @@ def extend_with_weight_decay(base_optimizer, name=None):
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_weight_decay_v2(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_weight_decay_v2(base_optimizer):
     """返回新的优化器类，加入权重衰减
     """
     class new_optimizer(base_optimizer):
@@ -389,14 +404,11 @@ def extend_with_weight_decay_v2(base_optimizer, name=None):
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_layer_adaptation(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_layer_adaptation(base_optimizer):
     """返回新的优化器类，加入层自适应学习率
     """
     class new_optimizer(base_optimizer):
@@ -440,20 +452,16 @@ def extend_with_layer_adaptation(base_optimizer, name=None):
 
         def get_config(self):
             config = {
-                'exclude_from_layer_adaptation':
-                self.exclude_from_layer_adaptation
+                'exclude_from_layer_adaptation': self.exclude_from_layer_adaptation
             }
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_layer_adaptation_v2(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_layer_adaptation_v2(base_optimizer):
     """返回新的优化器类，加入层自适应学习率
     """
     class new_optimizer(base_optimizer):
@@ -496,20 +504,16 @@ def extend_with_layer_adaptation_v2(base_optimizer, name=None):
 
         def get_config(self):
             config = {
-                'exclude_from_layer_adaptation':
-                self.exclude_from_layer_adaptation
+                'exclude_from_layer_adaptation': self.exclude_from_layer_adaptation
             }
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_piecewise_linear_lr(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_piecewise_linear_lr(base_optimizer):
     """返回新的优化器类，加入分段线性学习率
     """
     class new_optimizer(base_optimizer):
@@ -544,14 +548,11 @@ def extend_with_piecewise_linear_lr(base_optimizer, name=None):
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_piecewise_linear_lr_v2(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_piecewise_linear_lr_v2(base_optimizer):
     """返回新的优化器类，加入分段线性学习率
     """
     class new_optimizer(base_optimizer):
@@ -574,14 +575,11 @@ def extend_with_piecewise_linear_lr_v2(base_optimizer, name=None):
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_gradient_accumulation(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_gradient_accumulation(base_optimizer):
     """返回新的优化器类，加入梯度累积
     """
     class new_optimizer(base_optimizer):
@@ -636,14 +634,11 @@ def extend_with_gradient_accumulation(base_optimizer, name=None):
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_gradient_accumulation_v2(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_gradient_accumulation_v2(base_optimizer):
     """返回新的优化器类，加入梯度累积
     """
     class new_optimizer(base_optimizer):
@@ -691,14 +686,11 @@ def extend_with_gradient_accumulation_v2(base_optimizer, name=None):
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_lookahead(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_lookahead(base_optimizer):
     """返回新的优化器类，加入look ahead
     """
     class new_optimizer(base_optimizer):
@@ -749,14 +741,11 @@ def extend_with_lookahead(base_optimizer, name=None):
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_lookahead_v2(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_lookahead_v2(base_optimizer):
     """返回新的优化器类，加入look ahead
     """
     class new_optimizer(base_optimizer):
@@ -804,14 +793,11 @@ def extend_with_lookahead_v2(base_optimizer, name=None):
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_lazy_optimization(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_lazy_optimization(base_optimizer):
     """返回新的优化器类，加入懒惰更新
     """
     class new_optimizer(base_optimizer):
@@ -855,20 +841,16 @@ def extend_with_lazy_optimization(base_optimizer, name=None):
 
         def get_config(self):
             config = {
-                'include_in_lazy_optimization':
-                self.include_in_lazy_optimization
+                'include_in_lazy_optimization': self.include_in_lazy_optimization
             }
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-def extend_with_lazy_optimization_v2(base_optimizer, name=None):
+@export_to_custom_objects
+def extend_with_lazy_optimization_v2(base_optimizer):
     """返回新的优化器类，加入懒惰更新
     """
     class new_optimizer(base_optimizer):
@@ -908,53 +890,58 @@ def extend_with_lazy_optimization_v2(base_optimizer, name=None):
 
         def get_config(self):
             config = {
-                'include_in_lazy_optimization':
-                self.include_in_lazy_optimization
+                'include_in_lazy_optimization': self.include_in_lazy_optimization
             }
             base_config = super(new_optimizer, self).get_config()
             return dict(list(base_config.items()) + list(config.items()))
 
-    if is_string(name):
-        new_optimizer.__name__ = name
-        keras.utils.get_custom_objects()[name] = new_optimizer
-
     return new_optimizer
 
 
-class ExponentialMovingAverage(keras.callbacks.Callback):
-    """对模型权重进行指数滑动平均（作为Callback来使用）
+@export_to_custom_objects
+def extend_with_exponential_moving_average(base_optimizer):
+    """返回新的优化器类，加入EMA（权重滑动平均）
     """
-    def __init__(self, momentum=0.999):
-        self.momentum = momentum
-
-    def set_model(self, model):
-        """绑定模型，并初始化参数
+    class new_optimizer(base_optimizer):
+        """带EMA（权重滑动平均）的优化器
         """
-        super(ExponentialMovingAverage, self).set_model(model)
-        self.ema_weights = [K.zeros(K.shape(w)) for w in model.weights]
-        self.old_weights = K.batch_get_value(model.weights)
-        K.batch_set_value(zip(self.ema_weights, self.old_weights))
-        self.updates = []
-        for w1, w2 in zip(self.ema_weights, model.weights):
-            op = K.moving_average_update(w1, w2, self.momentum)
-            self.updates.append(op)
+        def __init__(self, ema_momentum=0.999, *args, **kwargs):
+            super(new_optimizer, self).__init__(*args, **kwargs)
+            self.ema_momentum = ema_momentum
 
-    def on_batch_end(self, batch, logs=None):
-        """每个batch后自动执行
-        """
-        K.batch_get_value(self.updates)
+        def get_updates(self, loss, params):
+            updates = super(new_optimizer, self).get_updates(loss, params)
+            self.model_weights = params
+            self.ema_weights = [K.zeros(K.shape(w)) for w in params]
+            self.old_weights = K.batch_get_value(params)
+            K.batch_set_value(zip(self.ema_weights, self.old_weights))
 
-    def apply_ema_weights(self):
-        """备份原模型权重，然后将平均权重应用到模型上去。
-        """
-        self.old_weights = K.batch_get_value(self.model.weights)
-        ema_weights = K.batch_get_value(self.ema_weights)
-        K.batch_set_value(zip(self.model.weights, ema_weights))
+            ema_updates, ema_momentum = [], self.ema_momentum
+            with tf.control_dependencies(updates):
+                for w1, w2 in zip(self.ema_weights, params):
+                    new_w = ema_momentum * w1 + (1 - ema_momentum) * w2
+                    ema_updates.append(K.update(w1, new_w))
 
-    def reset_old_weights(self):
-        """恢复模型到旧权重。
-        """
-        K.batch_set_value(zip(self.model.weights, self.old_weights))
+            return ema_updates
+
+        def get_config(self):
+            config = {'ema_momentum': self.ema_momentum}
+            base_config = super(new_optimizer, self).get_config()
+            return dict(list(base_config.items()) + list(config.items()))
+
+        def apply_ema_weights(self):
+            """备份原模型权重，然后将平均权重应用到模型上去。
+            """
+            self.old_weights = K.batch_get_value(self.model_weights)
+            ema_weights = K.batch_get_value(self.ema_weights)
+            K.batch_set_value(zip(self.model_weights, ema_weights))
+
+        def reset_old_weights(self):
+            """恢复模型到旧权重。
+            """
+            K.batch_set_value(zip(self.model_weights, self.old_weights))
+
+    return new_optimizer
 
 
 if is_tf_keras:
@@ -968,3 +955,10 @@ if is_tf_keras:
 else:
     Adam = keras.optimizers.Adam
     AdaFactor = AdaFactorV1
+
+custom_objects = {
+    'Adam': Adam,
+    'AdaFactor': AdaFactor,
+}
+
+keras.utils.get_custom_objects().update(custom_objects)
